;*******************************************************************************
;
;	Projet:
;	  Commande par PIC16F84 (ou PC16C84) pour un VFO DDS  AD9832 ( 0-8 MHz )
;
;	Fichier:
;	  ad9832_5.asm
;
;	Description:
;         La commande utilise un encodeur rotatif, un interrupteur de
;         verrouillage du VFO, un bouton poussoir de changement de pas, et
;         un LCD 1x16 de type HD44780.
;         Les donnes sries sont mises via 3 lignes  un chip DDS AD9832.
;	  Le LCD visualise la frquence en Hz ( par exemple 1.234.567 ) et
;         le pas utilis ( P=1, P=10, P=100,  P=1000, ou P=10 Khz ).
;         En fonction du niveau de la broche RA4, le LCD peut visualiser soit
;         la vraie frquence de sortie, soit la frquence de sortie divise 
;	  par 4. F/4 est utilis pour des rcepteurs  conversion directe,
;         o l'oscillateur local est divis par 4 pour obtenir des signaux
;         en quadrature.
;
;	  L'encodeur rotatif est chantillonn chaque 256sec. par un pointeur
;         ISR temps rel. Ceci incrmente/dcrmente la variable 24 bits "freq"
;         en fonction du sens de rotation de l'encodeur. Le Pas de frquence
;         est mmoris dans la variable 16 bits "step". Une pousse sur le bouton
; 	  fait varier les cycles entre 1, 10, 100 ,1000, et 10 Khz.
;
;         Une version avec un rapport de pas variable selon la vitesse de rotation
;	  de l'encodeur est plus lgante, mais rserve  des encodeurs de faible
;         rsolution tels que 32 encoches par tour.
;	  J'ai adopt l'utilisation de pas fixes avec des encodeurs Bourns de 100
; 	  ou de 256 encoches par tour.
;
;	  Lorsque le pointeur ISR modifie la valeur de "freq", il place le drapeau
;         "f_chang". Ce drapeau permet a la boucle principale de mettre  jour
;         le LCD et le chip DDS. En d'autres termes le LCD et le chip DDS ne seront
;	  mis  jour que lorsque la frquence aura t modifie. Ceci, afin de conserver
; 	  les rayonnements parasites au minimum. Mettre  jour le LCD et le DDS prend
;	  un peu de temps, mais ce n'est pas un problme car le pointeur ISR s'assure
;	  que la variable "freq" traque l'encodeur en permanence.
;
;	  L'interrupteur de verrouillage de la frquence est un interrupteur SPDT 
;	  3 positions, ON-OFF-(ON). La frquence est verrouille lorsque l'interrupteur
;	  est en position mdiane. La frquence est dverrouille en permanence sur la
;	  position ON (haute), et momentanment dverrouille sur la position ON (basse)
;
;       Dpendances:
;	  Le fichier p16f84.inc Rev.A (ou p16c84.inc Rev.C pour le C84)
;	  Dveloppement assembleur: Microchip/MPLAB 4.00 Released et PIC Start Plus
;
;	Notes:
;         Compilation par F5MI, JP Mallet,  partir de divers programmes concernant le
;	  chip AD9850, crits par F1BIU, WB2V, et SM6LKM
;
;	  Toute la chronomtrie est base sur l'emploi d'un Quartz de 4 Mhz.
;	  On peut galement commander le PIC par une horloge  3.125 Mhz drive de
;	  l'horloge 25 Mhz du DDS ( compteur F/8 ). Ceci peut aider  rduire les 
;         rayonnements parasites.
;
;	  Ce fichier peut tre assembl, soit pour un PIC16F84, soit pour un PIC16C84.
;	  Slectionner le processeur appropri et rassembler. Voir la section
;	  consacre aux directives d'assemblage. 
;
;	  Le code d'initialisation du LCD est crit pour un LCD 1x16, avec soit des
;	  fontes 5x7, soit des fontes 5x10. Slectionner la fonte approprie dans la
;	  section consacre aux directives d'assemblage et rassembler.
;
;	  Le bit RP0 du registre d'tat est mis  zro par dfaut ( RAM page 0 ).
;	  Le code qui place le bit RP0 doit le remettre  zro lorsqu'il a termin
;         d'accder  la RAM page 1.
;
;
;*******************************************************************************


;-------------------------------------------------------------------------------
;	Directives d'Assemblage
;-------------------------------------------------------------------------------

	title		"Commande de VFO DDS"
	subtitle	"AD9832"
	processor	16F84		; slecte 16F84 ou 16C84 *ICI seulement* !

#define	LCD5X10				; slecte LCD5X7 ou LCD5X10 ici!

	IFDEF	__16F84
	include		"p16f84.inc"	; ne pas toucher!
	ENDIF

	IFDEF	__16C84
	include		"p16c84.inc"	; ne pas toucher!
	ENDIF

	radix		dec		; radix dcimal par dfaut
	errorlevel	0		; montre tous les warnings et erreurs
	__idlocs	0xCAFE		; Pourquoi pas, c'est mieux que 0x2BAD...
	__config	_CP_OFF & _PWRTE_OFF & _WDT_OFF & _XT_OSC


;-------------------------------------------------------------------------------
;	Nom des variables RAM
;-------------------------------------------------------------------------------

	CBLOCK	0x0C	; on a 68(36) octets de RAM  partir de 0x0C - 0x2F(0x4F)
	  w_save	; W est sauv ici pendant l'xcution du pointeur ISR 
	  s_save	; STATUS est sauv ici pendant l'xcution du pointeur ISR 
	  enctmp	; variable temporaire pour utilisation dans le timer du pointeur ISR
	  prevenc	; L'tat de l'encodeur est sauv ici entre les pointeurs du timer
	  temp0		; variable temporaire utilise par de nombreuses routines
	  temp1
	  temp2
	  loopcnt	; compteur de boucle utilis par de nombreuses routines
	  freq0		; frquence 24-bit (Hz), freq0 est l'octet LS
	  freq1
	  freq2
	  bcd0		; 7-digits packed BCD, le nibble bas de bcd0 est le Digit LS
	  bcd1
	  bcd2
	  bcd3		; le nibble haut de bcd3 n'est pas utilis (8me digit)
	  dds0		; mot de contrle de frquence 32-bit pour le DDS AD9832 
	  dds1		; ...dds0 est l'octet LS
	  dds2
	  dds3
	  step0		; Pas de frquence 16-bit  (Hz/impulsion de l'encodeur)...
	  step1		; ...step0 est l'octet LS
	  bcdtmp	; variable temporaire utilise dans la convesion binaire-BCD 
	  osc0		; valeur de calibration 32-bit  = 2^56 / fclk (Hz)...
	  osc1		; ...osc0 est l'octet LS
	  osc2
	  osc3
	  osc_copy0	; osc est copi dans osc_copy avant multiplication...
	  osc_copy1	; ...parce que osc_copy sera dtruit dans...
	  osc_copy2	; ...le process.
	  osc_copy3
	  lcd_char	; utilis par les routines d'criture LCD 
	  flags		; divers bits drapeaux
	  freq_copy0	; calc_dds fait une copie atomique de freq vers...
	  freq_copy1	; ...freq_copy qui sera utilis plus tard (et dtruit)...
	  freq_copy2	; ...par bin2bcd

; Il y a 33 octets de RAM disponibles si on utilise un 'F84.
; Dans le cas d'un 'C84, il y a seulement un seul octet de disponible...
	ENDC


;	Nom des bits individuels de la variable "flags"

#define	f_lstep	flags,7		; dernier tat du BP de pas
#define	f_freg	flags,6		; utilis pour alterner entre les reg. f de l' AD9832 
#define	f_chang	flags,5		; drapeau freq. change  partir de l' ISR vers la boucle principale
#define	f_4	flags,4		; f_4  f_0 ne sont pas utiliss
#define	f_3	flags,3
#define	f_2	flags,2
#define	f_1	flags,1
#define	f_0	flags,0


;-------------------------------------------------------------------------------
;	Carte des broches I/O et macros lecture/criture des broches
;-------------------------------------------------------------------------------

;	broches du PORTA 

;	RA4	0=lit la frquence vraie, 1=lit freq/4 (Quadrature)
;	RA3	LCD D7, AD9832 SDATA
;	RA2	LCD D6, AD9832 SCLK
;	RA1	LCD D5
;	RA0	LCD D4


;	macros lecture/criture des broches du PORTA 

#define	i_div4	PORTA,4		; cette broche (si elle est utilise) ncessite une rsistance de pull-up
#define	o_sdata	PORTA,3
#define	o_sclk	PORTA,2
#define	o_ra1	PORTA,1
#define	o_ra0	PORTA,0


;	broches du PORTB (pullup interne est autorise pour toutes les entres du PORTB )

;	RB7	AD9832 FSYNC
;	RB6	LCD E
;	RB5	LCD RW
;	RB4	LCD RS
;	RB3	Interrupteur de verrouillage  la masse, 1 = verrouill
;	RB2	Bouton Poussoir de Pas  la masse
;	RB1	Encodeur phase I
;	RB0	Encodeur phase Q


;	macros lecture/criture du PORTB 

#define	o_fsync	PORTB,7
#define	o_e	PORTB,6
#define	o_rw	PORTB,5
#define	o_rs	PORTB,4
#define	i_lock	PORTB,3
#define	i_step	PORTB,2
#define	i_rb1	PORTB,1
#define	i_rb0	PORTB,0


;	direction des broches aprs la mise sous tension, 0=sortie, 1=entre

PUTRISA	equ	B'00010000'	; toutes les broches du PORTA except RA4 sont des sorties
PUTRISB	equ	B'00001111'	; le nibble haut du PORTB est en sortie


;	niveaux des broches de sortie aprs la mise sous tension

PUPORTA	equ	B'00000000'
PUPORTB	equ	B'10000000'	; AD9832 FSYNC , la broche est  l'tat haut au repos

;-------------------------------------------------------------------------------
;	Point d'entre du Reset 
;-------------------------------------------------------------------------------

	ORG	0x0000

	goto	init


;-------------------------------------------------------------------------------
;	Pointeur temps rel ISR (256 s = 3906.25 Hz)
;
;	L'ISR (routine de service d'interruption) lance un dcodeur en quadrature
;	qui surveille l'entre de l'encodeur.
;
;	La valeur 24-bit "freq" est incrmente/dcrmente par
;	la valeur 16-bit "step" qui dpend du sens de rotation de l'encodeur.
;	Le rsultat dans "freq" est bloqu  0 et  6 MHz.
;
;	Supposons 50% de cycle de travail, et exactement 90 degrs de dphasage entre
;	les canaux de l'encodeur, la vitesse maximum du bouton de commande
;	est 1/(PPR*1024e-6) tours/s (PPR est le nombre d'impulsions de
;	l'encodeur par tour).
;
;	Exemple: Un encodeur 100PPR pourra tourner jusqu' 9.77 tour par seconde
;	 (585 RPM) sans manquer un seul pas. Comme le dcodeur en quadrature compte
;	chaque ct de l'encoche, un encodeur  100 PPR donnera 400 pas de frquence par tour.
;-------------------------------------------------------------------------------

	ORG	4

	movwf	w_save		; sauve les registres importants
	swapf	STATUS,w
	bcf	_RP0
	movwf	s_save
	bcf	_T0IF		; RAZ du drapeau d'interruption TMR0 

;	Check encoder

	movf	PORTB,w		; Lit l'tat courant de l'encodeur dans...
	movwf	enctmp		; ...enctmp (bits 0 et 1)

	bcf	enctmp,2	; copie l'ancien tat de l'encodeur dans...
	btfsc	prevenc,0	; ...enctmp (bits 2 and 3)
	bsf	enctmp,2
	bcf	enctmp,3
	btfsc	prevenc,1
	bsf	enctmp,3

	movwf	prevenc		; sauve l'tat courant de l'encodeur pour le pointeur suivant

	btfsc	i_lock		; VFO verrouill?
	goto	enc_done	; oui, saute inc/dc 

	movf	enctmp,w	; ralise la combinaison de l'ancien tat et de l'actuel
	andlw	0x0F		; fait le mnage du nibble suprieur
	addwf	PCL,f		; saute dans la table ci-dessous

	goto	enc_done	; 00 -> 00 = pas de changement
	goto	enc_inc		; 00 -> 01 = incrmente
	goto	enc_dec		; 00 -> 10 = dcrmente
	goto	enc_done	; 00 -> 11 = illgal, ignorer
	goto	enc_dec		; 01 -> 00 = dcrmente
	goto	enc_done	; 01 -> 01 = pas de changement
	goto	enc_done	; 01 -> 10 = illgal, ignorer
	goto	enc_inc		; 01 -> 11 = incrmente
	goto	enc_inc		; 10 -> 00 = incrmente
	goto	enc_done	; 10 -> 01 = illgal, ignorer
	goto	enc_done	; 10 -> 10 = pas de changement
	goto	enc_dec		; 10 -> 11 = dcrmente
	goto	enc_done	; 11 -> 00 = illgal, ignore
	goto	enc_dec		; 11 -> 01 = dcrmente
	goto	enc_inc		; 11 -> 10 = incrmente
	goto	enc_done	; 11 -> 11 = pas de changement

enc_dec:
	bsf	f_chang		; Appel de la boucle principale pour mettre  jour LCD & DDS
	movf	step0,w		; freq -= step
	subwf	freq0,f
	movf	step1,w
	btfss	_C
	incfsz	step1,w
	subwf	freq1,f
	btfss	_C
	decf	freq2,f

	btfss	freq2,7		; frquence ngative ?
	goto	enc_done

	clrf	freq0		; oui, bloquer  0 Hz
	clrf	freq1
	clrf	freq2
	goto	enc_done

enc_inc:
	bsf	f_chang		; Appel de la boucle principale pour mettre  jour LCD & DDS
	movf	step0,w		; freq += step
	addwf	freq0,f
	movf	step1,w
	btfsc	_C
	incfsz	step1,w
	addwf	freq1,f
	btfsc	_C
	incf	freq2,f

;	Si freq > 8 MHz alors placer freq = 8 MHz (0x7A1200)

	movf	freq2,w
	sublw	0x7A		; freq2 > 0x7A?
	btfss	_C
	goto	setmax		; oui, bloquer la valeur

	btfss	_Z		; freq2 == 0x7A?
	goto	enc_done	; non, freq2 < 0x7A, c'est correct

	movf	freq1,w
	sublw	0x12		; freq1 > 0x12?
	btfss	_C
	goto	setmax		; oui, bloquer la valeur
	btfss	_Z		; freq1 == 0x12?
	goto	enc_done	; non, freq1 < 0x12, c'est correct

	movf	freq0,w
	sublw	0x00		; freq0 > 0x00?
	btfsc	_C
	goto	enc_done	; non, freq0 <= 0x00, c'est correct

setmax:
	movlw	0x7A		; Placer freq = 8 MHz
	movwf	freq2
	movlw	0x12
	movwf	freq1
	movlw	0x00
	movwf	freq0

enc_done:
	swapf	s_save,w	; Restaurer les registres importants
	movwf	STATUS
	swapf	w_save,f
	swapf	w_save,w

	retfie


;-------------------------------------------------------------------------------
;	Initialisation
;-------------------------------------------------------------------------------

init:

;	Placer les latches des broches de sortie

	movlw	PUPORTA
	movwf	PORTA
	movlw	PUPORTB
	movwf	PORTB

;	Placer la direction des broches et mettre le registre OPTION  l'tat haut

	bsf	_RP0		; slecte RAM page 1
	movlw	PUTRISA		; directions des broches pour le PORTA
	movwf	0x7F & TRISA
	movlw	PUTRISB		; directions des broches pour le PORTB
	movwf	0x7F & TRISB
	movlw	B'01011111'	; PORTB pullup, clk TMR0 interne , PS en WDT
	movwf	0x7F & OPTI
	bcf	_RP0		; RAM page 0 est la page par dfaut

;	Initialise le LCD, le controleur HD44780, en mode 4-bit 
;	(E, R/W et RS sont bas  ce moment)

	call	delay100	; alloue 100 ms pour la mise sous tension du LCD 

	movlw	0x30		; function set, interface 8-bit 
	movwf	PORTA
	bsf	o_e		; faire l'impulsion E 
	bcf	o_e
	call	delay100	; on peut tester le bit busy 

	movlw	0x30		;  nouveau...
	movwf	PORTA
	bsf	o_e
	bcf	o_e
	call	delay100

	movlw	0x30		; ...et encore
	movwf	PORTA
	bsf	o_e
	bcf	o_e
	call	delay100

	movlw	0x20		; maintenant quand on est sur que l'on a ...
	movwf	PORTA		; ...une interface 8-bit ...
	bsf	o_e		; ...on peut procder  la slection...
	bcf	o_e		; ...de l'interface 4-bit en lieu et place de l'autre (hmmm...)
	call	delay100

	IFDEF LCD5X7
	  movlw	  0x20		; interface 4-bit, 1 ligne, fonte 5x7 
	ELSE
	  IFDEF	LCD5X10
	    movlw   0x24	; interface 4-bit, 1 ligne, fonte 5x10
	  ELSE
	    ERROR "Aucune fonte valide slectione!"
	  ENDIF
	ENDIF
	call	lcd_ctrl

	movlw	0x01		; RAZ de la visu, curseur  gauche
	call	lcd_ctrl
	movlw	0x06		; mode incrmentation, la visu ne se dcale pas
	call	lcd_ctrl
	movlw	0x0C		; visu en service, pas de curseur, pas de scintillement
	call	lcd_ctrl

;	Rgle la frquence initiale

	movlw	0x02		; 135.7 kHz
	movwf	freq2
	movlw	0x12
	movwf	freq1
	movlw	0x14
	movwf	freq0

;	Place le facteur de calibration

	movlw	0xAB		; Place osc = 2^56 / fclk, 25 MHz => 0xABCC7712
	movwf	osc3		; Note! La valeur sera sauve en EEPROM...
	movlw	0xCC		; ...lors d'une version future du programme
	movwf	osc2
	movlw	0x77
	movwf	osc1
	movlw	0x12
	movwf	osc0

;	Initialize AD9832 chip

	movlw	0xF8		; SLEEP = RESET = CLR = 1
	movwf	temp1		; (L'octet LS envoy depuis temp0 n'est pas important)
	call	send_dds
	movlw	0xB0		; SYNC = SELSRC = 1
	movwf	temp1		; (L'octet LS envoy depuis temp0 n'est pas important)
	call	send_dds
	movlw	0xC0		; SLEEP = RESET = 0
	movwf	temp1		; (L'octet LS envoy depuis temp0 n'est pas important)
	call	send_dds

;	Initialise le travail de l'encodeur

	movf	PORTB,w		; Evite des Pas parasites de l'encodeur  la mise sous-tension
	movwf	prevenc

	bsf	_GIE		; autorise l'interruption globale (bit T0IE ...
				; ...sera plac dans la boucle principale)
;				; Ici on va procder  la....

	bsf	f_chang		; premire mise  jour de LCD & DDS sans toucher au bouton de commande
	movlw	0xC2		; Mettre (invisible) le curseur au caractre 11
	call	lcd_ctrl
	goto	set_step_1h	; Placer le pas initial  1 Hz...


;-------------------------------------------------------------------------------
;	Boucle principale du programme
;-------------------------------------------------------------------------------

mainlooptop:

;	Teste le BP de pas, temps d'xcution de la boucle principale
;	est utilis pour enlever les rebonds. Peut tre trop petite si
;	le bouton poussoir a des rebonds!".

	btfss	i_step		; BP Pas, press?
	goto	chkedge

	bsf	f_lstep		; non, conserver ce drapeau mis  jour pour le prochain test
	goto	compute

chkedge:
	btfss	f_lstep		; oui, Cette action survient-elle depuis le dernier test?
	goto	clr_lstep

;	Cycle au travers des pas utilisables
;	1 Hz - 10 Hz - 100 Hz - 1 kHz - 10 kHz - 1 Hz etc.

change_step:
	movlw	0xC2		; oui, placer le curseur au caractre 11
	call	lcd_ctrl
	bcf	_T0IE		; interruption TMR0 off (mise  jour atomique de step)
	btfsc	step0,0		; Pas = 1 Hz? (0x0001)
	goto	set_step_10h
	btfsc	step0,1		; Pas = 10 Hz? (0x000A)
	goto	set_step_100h
	btfsc	step0,2		; Pas = 100 Hz? (0x0064)
	goto	set_step_1k
	btfsc	step0,7		; Pas = 1 kHz? (0x03E8)
	goto	set_step_10k

;	Le pas semble tre de 10 kHz, placer le pas  1 Hz

set_step_1h:
	clrf	step1		; Pas = 0x0001 = 1
	movlw	0x01
	movwf	step0
	bsf	_T0IE		; dmasquer l'intrruption TMR0 
	movlw	32		; crire "  P=1"
	call	lcd_data
	movlw	32
	call	lcd_data
	movlw	'P'
	call	lcd_data
	movlw	'='
	call	lcd_data
	movlw	'1'
	call	lcd_data
	goto	clr_lstep

set_step_10h:
	clrf	step1		; Pas = 0x000A = 10
	movlw	0x0A
	movwf	step0
	bsf	_T0IE		; dmasquer l'interruption TMR0 
	movlw	32		; crire " P=10"
	call	lcd_data
	movlw	'P'
	call	lcd_data
	movlw	'='
	call	lcd_data
	movlw	'1'
	call	lcd_data
	movlw	'0'
	call	lcd_data
	goto	clr_lstep

set_step_100h:
	clrf	step1		; Pas = 0x0064 = 100
	movlw	0x64
	movwf	step0
	bsf	_T0IE		; dmasquer l'interruption TMR0 
	movlw	'P'		; crire "P=100"
	call	lcd_data
	movlw	'='
	call	lcd_data
	movlw	'1'
	call	lcd_data
	movlw	'0'
	call	lcd_data
	movlw	'0'
	call	lcd_data
	goto	clr_lstep

set_step_1k:
	movlw	0x03		; Pas = 0x03E8 = 1000
	movwf	step1
	movlw	0xE8
	movwf	step0
	bsf	_T0IE		; dmasquer l'interruption TMR0 
	movlw	32		; crire " P=1k"
	call	lcd_data
	movlw	'P'
	call	lcd_data
	movlw	'='
	call	lcd_data
	movlw	'1'
	call	lcd_data
	movlw	'k'
	call	lcd_data
	goto	clr_lstep

set_step_10k:
	movlw	0x27		; Pas = 0x2710 = 10000
	movwf	step1
	movlw	0x10
	movwf	step0
	bsf	_T0IE		; dmasquer l'interruption TMR0 
	movlw	'P'		; crire "P=10k"
	call	lcd_data
	movlw	'='
	call	lcd_data
	movlw	'1'
	call	lcd_data
	movlw	'0'
	call	lcd_data
	movlw	'k'
	call	lcd_data
clr_lstep:
	bcf	f_lstep		; conserver ce drapeau mis  jour pour le prochain test

compute:
	btfss	f_chang		; La frquence a t'elle change?
	goto	dummydelay	; non, attendre un peu

	bcf	f_chang		; oui, RAZ le changement de drapeau 
	call	calc_dds	; faire freq_copy, calculer la mot de frquence du DDS 
	call	load_dds	; envoyer le mot de frquence du DDS  l' AD9832
	call	bin2bcd		; convertir freq_copy en BCD
	call	showfreq	; montrer la frquence BCD sur le LCD
	goto	mainlooptop

;	Attendre environ 10 ms pour viter le rebond du BP Pas mme
;	s'il n'y a aucune nouvelle frquence  calculer.

dummydelay:
	movlw	13
	movwf	temp0
dummydel1:
	movlw	255
	movwf	temp1
dummydel2:
	decfsz	temp1,f
	goto	dummydel2
	decfsz	temp0,f
	goto	dummydel1
	goto	mainlooptop

;-------------------------------------------------------------------------------
;	Conversion de la frquence (Hz) en mot DDS en utilisant un facteur de calibration de l' osc.
;	Cette "version QRP" de la routine de multiplication de WB2V utilise seulement 24 bits pour
;	la frquence (Hz). Le mot du DDS a t rtrci de 40  32 bits. De toute faon la valeur de
;	l'oscillateur est toujours un nombre  32 bits.
;
;	Cette routine fait une copie atomique de freq  freq_copy.
;	La copie est prserve pendant la multiplication, et elle est utilise plus tard par la routine
;	bin2bcd.
;-------------------------------------------------------------------------------

calc_dds:
	bcf	_T0IE		; Cette opration de copie doit tre atomique...
	movf	freq0,w		; ...de faon  viter que la valeur de la frquence soit...
	movwf	freq_copy0	; ...mise  jour par ISR pendant...
	movf	freq1,w		; ...la multiplication
	movwf	freq_copy1
	movf	freq2,w
	movwf	freq_copy2
	bsf	_T0IE		; re-autorise le pointeur d'interruption

	clrf	dds0		; RAZ l'accumulateur de multiplication
	clrf	dds1
	clrf	dds2
	clrf	dds3
	movlw	32		; initialise le compteur de boucle de multiplication
	movwf	loopcnt
	movf	osc0,w		; copie osc vers osc_copy car la valeur...
	movwf	osc_copy0	; ...sera dtruite dans le processus...
	movf	osc1,w
	movwf	osc_copy1
	movf	osc2,w
	movwf	osc_copy2
	movf	osc3,w
	movwf	osc_copy3
multloop:
	bcf	_C
	btfss	osc_copy0,0	; additionne le poids de ce bit osc?
	goto	shiftit

	movf	freq_copy0,w	; dds += (freq_copy << 8)
	addwf	dds1,f
	movf	freq_copy1,w
	btfsc	_C
	incfsz	freq_copy1,w
	addwf	dds2,f
	movf	freq_copy2,w
	btfsc	_C
	incfsz	freq_copy2,w
	addwf	dds3,f
shiftit:
	rrf	dds3,f		; aligne dds et osc_copy pour...
	rrf	dds2,f		; ...le prochain poids binaire
	rrf	dds1,f
	rrf	dds0,f
	rrf	osc_copy3,f
	rrf	osc_copy2,f
	rrf	osc_copy1,f
	rrf	osc_copy0,f
	decfsz	loopcnt,f	; faire?
	goto	multloop

	return


;-------------------------------------------------------------------------------
;	Mise  jour du chip DDS par un nouveau mot de frquence
;
;	Note! Comme il n'y a aucun moyen de mettre  jour simultanment tous les 32 bits 
;	dans l'AD9832, on basculera entre les deux registres de frquence.
;	Exemple: Load reg0, switch to reg0, load reg1, switch
;	to reg1 etc. etc. Ceci pour prvenir les problmes survenant lorsque deux octets ou plus
;	changent de valeur en mme temps.
;	(Voir la data sheet de l'AD9832...)
;-------------------------------------------------------------------------------

load_dds:
	btfss	f_freg		; le registre de frquence 0 a t'il t mis  jour la dernire fois?
	goto	load_reg1	; oui, mettre  jour le registre 1 cette fois i.

	bcf	f_freg		; mise  jour du registre 0 cette fois, 1 la fois suivante
	movlw	0x30		; crire pour ajourner le registre, destination = 0
	movwf	temp1
	movf	dds0,w		; octet bas
	movwf	temp0
	call	send_dds	; appeler le hardware

	movlw	0x21		; crire 16 bits de freq, destination = 1
	movwf	temp1
	movf	dds1,w		; octet milieu bas
	movwf	temp0
	call	send_dds	; appeler le hardware

	movlw	0x32		; crire pour ajourner le registre, destination = 2
	movwf	temp1
	movf	dds2,w		; octet milieu haut
	movwf	temp0
	call	send_dds	; appeler le hardware

	movlw	0x23		; crire 16 bits de freq, destination = 3
	movwf	temp1
	movf	dds3,w		; octet haut
	movwf	temp0
	call	send_dds	; appeler le hardware

	movlw	0x50		; commuter sur le registre de frquence 0
	movwf	temp1           ; (les donnes de temp0 ne sont pas importantes)
	call	send_dds	; Appeler le hardware
	return

load_reg1:
	bsf	f_freg		; mise  jour du registre 1 cette fois, 0 la fois suivante
	movlw	0x34		; crire pour ajourner le registre, destination = 4
	movwf	temp1
	movf	dds0,w		; octet bas
	movwf	temp0
	call	send_dds	; appeler le hardware

	movlw	0x25		; crire 16 bits de freq, destination = 5
	movwf	temp1
	movf	dds1,w		; octet milieu bas
	movwf	temp0
	call	send_dds	; Appeler le hardware

	movlw	0x36		; crire pour ajourner le registre, destination = 6
	movwf	temp1
	movf	dds2,w		; octet milieu haut
	movwf	temp0
	call	send_dds	; Appeler le hardware

	movlw	0x27		; crire 16 bits de freq, destination = 7
	movwf	temp1
	movf	dds3,w		; octet haut
	movwf	temp0
	call	send_dds	; Appeler le hardware

	movlw	0x58		; commuter sur le registre de frquence 1
	movwf	temp1           ; (les donnes de temp0 ne sont pas importantes)
	call	send_dds	; Appeler le hardware
	return


;-------------------------------------------------------------------------------
;	Envoi d'un mot de 16-bits (temp1:temp0) au chip DDS 
;-------------------------------------------------------------------------------

send_dds:
	movlw	16
	movwf	loopcnt
	bcf	o_fsync		; mettre la broche FSYNC de l'AD9832 basse
send_loop:
	bcf	o_sdata		; suppose bit zro, broche SDATA de l'AD9832  = 0
	btfsc	temp1,7		; Tester le Bit MSdu mot de 16-bits 
	bsf	o_sdata		; Si c'tait un 1, la broche SDATA de l'AD9832 = 1
	bsf	o_sclk		; envoyer une impulsion sur la broche SCLK de l'AD9832 
	bcf	o_sclk
	rlf	temp0,f		; rotation de tous les 16 bits  gauche (au travers de la retenue)
	rlf	temp1,f
	decfsz	loopcnt,f
	goto	send_loop

	bsf	o_fsync		; mettre la broche FSYNC de l'AD9832 haute
	return


;-------------------------------------------------------------------------------
;	Visualiser la frquence sur le LCD ( les 7 digits les plus  droite dans le tampon bcd )
;-------------------------------------------------------------------------------

showfreq:
	movlw	0x80		; Placer le curseur  la position la plus  gauche
	call	lcd_ctrl

	movf	bcd3,w		; lire le digit BCD 1M dans le nibble bas de W
	andlw	0x0F		; masquer le nibble bas
	addlw	0x30		; additionner le dcalage ASCII
	call	lcd_data	; Envoyer au LCD

	movlw	'.'
	call	lcd_data	; Point entre les digits 1M et 100k 

	swapf	bcd2,w		; lire le digit 100k BCD dans le nibble bas de W
	andlw	0x0F		; masquer le nibble bas
	addlw	0x30		; additionner le dcalage ASCII 
	call	lcd_data	; Envoyer au LCD LCD

	movf	bcd2,w		; lire le digit 10k BCD dans le nibble bas de W
	andlw	0x0F		; masquer le nibble bas
	addlw	0x30		; additionner le dcalage ASCII 
	call	lcd_data	; Envoyer au LCD

	swapf	bcd1,w		; lire le digit 1k BCD digit dans le nibble bas de W
	andlw	0x0F		; masquer le nibble bas
	addlw	0x30		; additionner le dcalage ASCII
	call	lcd_data	; Envoyer au LCD

	movlw	'.'
	call	lcd_data	; Point entre les digits 1k et 100H 

	movf	bcd1,w		; lire le digit 100H BCD dans le nibble bas de W
	andlw	0x0F		; masquer le nibble bas
	addlw	0x30		; additionner le dcalage ASCII
	call	lcd_data	; Envoyer au LCD

	swapf	bcd0,w		; lire le digit 10H BCD dans le nibble bas de W
	andlw	0x0F		; masquer le nibble bas
	addlw	0x30		; additionner le dcalage ASCII 
	call	lcd_data	; Envoyer au LCD

	movlw	0xC0		; Placer le curseur au caractre 9
	call	lcd_ctrl

	movf	bcd0,w		; lire le digit 1H BCD dans le nibble bas de W
	andlw	0x0F		; masquer le nibble bas
	addlw	0x30		; additionner le dcalage ASCII
	call	lcd_data	; Envoyer au LCD
	return


;-------------------------------------------------------------------------------
;	Conversion 24-bits binaire--BCD.
;
;	Les donnes d'entre binaires sont prleves dans freq_copy qui est dtruit.
;	Le rsultat est sauv dans bcd0..bcd3, bcd0 mmorise les 2 Digits LS.
;
;	C'est une version 24 bits modifie de la routine 32 bits de WB2V.
;	La valeur d'entre N'EST PAS prserve en plaant les bits dcals dans le bit 0
;	(comme ralis dans la version initiale de WB2V).
;-------------------------------------------------------------------------------

bin2bcd:
	btfss	i_div4		; divise la lecture par 4?
	goto	do_b2b		; non

	rrf	freq_copy2,f	; oui, dcaler la valeur binaire de 2 bits vers la droite
	rrf	freq_copy1,f
	rrf	freq_copy0,f
	rrf	freq_copy2,f
	rrf	freq_copy1,f
	rrf	freq_copy0,f
	movlw	0x3F
	andwf	freq_copy2,f	; Mettre de l'ordre
do_b2b:
	movlw	24		; le RAZ de la retenue est-il rellement ncessaire ici?...
	movwf	loopcnt		; ...a semble marcher sans cela
	clrf	bcd0
	clrf	bcd1
	clrf	bcd2
	clrf	bcd3
bcdloop:
	rlf	freq_copy0,f	; copie rotative de freq au travers de la retenue...
	rlf	freq_copy1,f
	rlf	freq_copy2,f
	rlf	bcd0,f		; ...dans bcd
	rlf	bcd1,f
	rlf	bcd2,f
	rlf	bcd3,f
	decfsz	loopcnt,f
	goto	adjust
	return

adjust:
	movlw	bcd0		; faire un ajustement magique pour chaque digit BCD !
	movwf	FSR
	call	adjbcd
	incf	FSR,f
	call	adjbcd
	incf	FSR,f
	call	adjbcd
	incf	FSR,f
	call	adjbcd
	goto	bcdloop

adjbcd:
	movlw	0x03		; cette version originale de adjbcd ( partir de la note...
	addwf	INDF,w		; ...MicroChip's AN526 travaille trs bien ici...
	movwf	bcdtmp		; ...tandis que la version de WB2Vne le fait pas!...
	btfsc	bcdtmp,3	; ...Explication?
	movwf	INDF
	movlw	0x30
	addwf	INDF,w
	movwf	bcdtmp
	btfsc	bcdtmp,7
	movwf	INDF
	return


;-------------------------------------------------------------------------------
;	Attendre que le LCD soit prt  recevoir des donnes
;-------------------------------------------------------------------------------

busywait:
	bsf	_RP0
	movlw	0x1F		; 3 tats RA0..RA3
	movwf	0x7F & TRISA
	bcf	_RP0
	bcf	o_rs		; slecte le registre de commande LCD 
	bsf	o_rw		; lance le LCD en lecture
busyloop:
	bsf	o_e		; E monte
	movf	PORTA,w		; lit le nibble suprieur, le bit busy est le bit 3 de W
	bcf	o_e		; E baisse
	bsf	o_e		; E monte, lecture bidon du nibble infrieur
	bcf	o_e		; E baisse
	andlw	0x08		; W &= 0x08, masque le bit busy 
	btfss	_Z		; zro?
	goto	busyloop	; non, garder en attente

	bcf	o_rw		; lance le LCD en criture
	bsf	_RP0
	movlw	0x10		; autorise  nouveau RA0..RA3 
	movwf	0x7F & TRISA
	bcf	_RP0
	return


;-------------------------------------------------------------------------------
;	Envoi de l'octet donnes/commande de W au LCD
;-------------------------------------------------------------------------------

lcd_ctrl:
	movwf	lcd_char	; sauve l'octet  envoyer
	call	busywait
	bcf	o_rs		; slecte le registre de commande du LCD 
	goto	lcd_out		; utilise le code i-dessous

lcd_data:
	movwf	lcd_char	; sauve l'octet  envoyer
	call	busywait
	bsf	o_rs		; slecte le registre de donnes du LCD 
lcd_out:
	swapf	lcd_char,w	; crit le nibble MS de donnes dans RA0..RA3
	movwf	PORTA
	bsf	o_e		; fait l'impulsion E 
	bcf	o_e
	movf	lcd_char,w	; crit le nibble LS de donnes dans RA0..RA3
	movwf	PORTA
	bsf	o_e		; fait l'impulsion E 
	bcf	o_e
	return


;-------------------------------------------------------------------------------
;	Sous-programme de retard de 100 ms  (100 ms incluent appel & retour)
;-------------------------------------------------------------------------------

delay100:
	movlw   198		; 99996 cycles de retard dmarrent ici
	movwf	temp0
d100_1:
	movlw   167
	movwf   temp1
d100_2:
	decfsz  temp1,F
	goto    d100_2
	decfsz  temp0,F
	goto    d100_1
	goto    $+1
	goto    $+1
	nop
	return

;-------------------------------------------------------------------------------
	END
